# Description: extend and validate a list of options
# Input:
#   type: type of command being processed (for error reporting only)
#   options: a named list comprising the options
#   required: a character vector containing the names of the required arguments
#   requiredTypes: a character vector of types ('c','n','l', 'cv') matching 'required'
#   optional: a character vector containing the names of the optional arguments
#   optionalTypes: a character vector of types ('c','n','l') matching 'optional'
#   defaults: a named list of defaults for some of the optional elements
#
# Value: a named list containing all required options, supplied optional options, and
#        and optional options with defaults (possible overridden)
#
# Details: variable types can be "n" for numeric, 
#          "c" for character, "cv" for character vector, and "l" for logical
parseOptions = function(type, options, required, requiredTypes, 
                        optional, optionalTypes, lineNumOffset, defaults=NULL) {
  
  # Check that only valid options are included and that all required options are present
  optNames = names(options)
  bad = setdiff(optNames, c(required, optional))
  if (length(bad)>0) {
    stop("@", lineNumOffset, " \\", type, " cannot include option(s): ", paste(bad, collapse=", "))
  }
  missingReq = setdiff(required, optNames)
  if (length(missingReq)>0) {
    stop("@", lineNumOffset, " \\", type, " requires option(s): ", paste(missingReq, collapse=", "))
  }
  
  # Construct vector of variable types matching 'options'
  nOpts = length(optNames)
  optTypes = rep(NA_character_, nOpts)
  optTypes[match(required, optNames)] = requiredTypes
  temp = match(optional, optNames)
  Sel = !is.na(temp)
  optTypes[match(optional[Sel], optNames)] = optionalTypes[Sel]
  assure(all(!is.na(optTypes)), 70026)
  
  #  Check data types of options and convert "cv" type to vectors
  for (opt in 1:nOpts) {
    oType = optTypes[opt]
    if (oType == "n") {
      check = suppressWarnings(as.numeric(options[[opt]]))
      bad = is.na(check)
      if (any(bad)) {
        stop("@", lineNumOffset, ' \\', oType, ' has non-numeric ', names(options)[opt])
      }
      options[[opt]] = check
    } else if (oType == "nv") {
      temp = strsplit(options[[opt]], ",")[[1]]
      check = sapply(temp, function(x) suppressWarnings(as.numeric(x)))
      bad = any(is.na(check))
      if (any(bad)) {
        stop("@", lineNumOffset, ' \\', type, ' has non-numeric ', names(options)[opt])
      }
      options[[opt]] = temp
    } else if (oType == "l") {
      check = suppressWarnings(as.logical(options[[opt]]))
      bad = is.na(check)
      if (any(bad)) {
        stop("@", lineNumOffset, ' \\', type, ' has non-logical ', names(options)[opt])
      }
      options[[opt]] = check
    } else if (oType == "cv") {
      options[[opt]] = sapply(strsplit(options[[opt]], ",")[[1]], str_trim)
    } else if (oType == "c") {
    } else {
      assure(FALSE, 221983)
    }
  } # end for each supplied option
  
  # Add in defaults
  neededDefaults = setdiff(names(defaults), names(options))
  if (!is.null(neededDefaults)) options = c(options, defaults[neededDefaults])
  
  return(options)
} # end parseOptions()



# Description: drop extra commas (which cause 'argument is missing, with no default' errors)
# Arguments: 
#   lst: a doList (see startDoList())
# Value: the same 'lst' with extra commas removed from the uiCode elements
#
dropExtraCommas = function(lst) {
  #browser()
  # get true order
  m = length(lst)
  Ord = c(1, rep(NA, m-1))
  for (i in 2:m) Ord[i] = lst[[Ord[i-1]]]$nextElement

  # fixup ui.r based on indenting pattern
  indentInOrder = sapply(lst[Ord], function(x) x$indent)
  toFix = Ord[which(indentInOrder[1:(m-1)] > indentInOrder[2:m])]
  for (i in toFix) {
    n = length(lst[[i]]$uiCode)
    nn = nchar(lst[[i]]$uiCode[n])
    lst[[i]]$uiCode[n] = substring(lst[[i]]$uiCode[n], 1, nn-1)
  }
    
  return(lst)
} # end dropExtraCommas()
