# Parse one line shinyTex commands (e.g., \shinySlider)
#
# Arguments:
#   text: text containing one or more commands (but only one per line, and one line per command)
#
# Value:
#   Shiny ui code for each command
#
processCommands = function(text, lineNumOffset) {
  rtn = NULL
  n = length(text)
  for (line in 1:n) {
    temp = parseCommand(text[line], lineNumOffset+line-1)
    type = temp$type
    options = temp$options
    
    # process command
    if (type == "shinySlider") {
      rtn = c(rtn, makeCall(type, "sliderInput", options, 
                            required = c("inputId", "label", "min", "max", "value"),
                            requiredTypes = c("c","c","n","n","n"),
                            optional = c("step", "round", "ticks", "animate", "width", "sep", "pre", "post"),
                            optionalTypes = c("n","n","l","l","c","c","c","c"),
                            lineNumOffset=lineNumOffset))
      
    } else if (type == "shinyDropdown") {
      rtn = c(rtn, makeCall(type, "selectInput", options, 
                              required = c("inputId", "label", "choices"),
                              requiredTypes = c("c","c","cv"),
                              optional = c("selected", "multiple", "selectize", "width", "size"),
                              optionalTypes = c("c","l","l","c","n"),
                              lineNumOffset=lineNumOffset))
        
    } else if (type == "shinyNumericInput") {
      rtn = c(rtn, makeCall(type, "numericInput", options, 
                            required = c("inputId", "label", "value"),
                            requiredTypes = c("c","c","n"),
                            optional = c("min", "max", "step"),
                            optionalTypes = c("n","n","n"),
                            lineNumOffset=lineNumOffset))
      
    } else if (type == "shinyActionButton") {
      rtn = c(rtn, makeCall(type, "actionButton", options, 
                            required = c("inputId", "label"), requiredTypes = c("c","c"),
                            optional = "icon", optionalTypes = "c",
                            lineNumOffset=lineNumOffset))

    } else if (type == "shinyRadioButtons") {
      rtn = c(rtn, makeCall(type, "radioButtons", options, 
                            required = c("inputId", "label", "choices"), requiredTypes = c("c","c","cv"),
                            optional = c("selected","inline"), optionalTypes = c("c","l"),
                            lineNumOffset))
      
    } else if (type == "shinyCheckbox") {
      rtn = c(rtn, makeCall(type, "checkboxInput", options, 
                            required = c("inputId", "label", "value"), requiredTypes = c("c","c","l"),
                            optional = NULL, optionalTypes = NULL,
                            lineNumOffset))
      
    } else if (type == "shinyTextInput") {
      rtn = c(rtn, makeCall(type, "textInput", options, 
                            required = c("inputId", "label"), requiredTypes = c("c","c"),
                            optional = "value", optionalTypes = "c",
                            lineNumOffset))
      
    } else if (type == "shinyQuiz") {
      rtn = c(rtn, makeCall(type, "quiz", options, 
                            required = c("bank"), requiredTypes = "c",
                            optional = c("addNumbers","randomizeQuestions","randomizeAnswers", "max", "interElement"), 
                            optionalTypes = c("l","l","l","n","c"), lineNumOffset))
      
    } else if (type == "shinyImage") {
      options = parseOptions(type, options, 
                             required = c("filename"), requiredTypes = "c",
                             optional = c("width","height"), 
                             optionalTypes = c("n","n"), lineNumOffset=lineNumOffset)
      str = paste0('HTML("<img src=\'', options$filename, '\'',
                   ifelse(any(names(options)=="width"), paste0("width='", options$width, "' "), ""),
                   ifelse(any(names(options)=="height"), paste0("height='", options$height, "' "), ""),
                   '>"),')
      rtn = c(rtn, str)
      
    } else if (type == "shinyVideo") {
      options = parseOptions(type, options, 
                             required = c("filename"), requiredTypes = "c",
                             optional = c("width","height","type","autoplay"), 
                             optionalTypes = c("n","n","c","l"), lineNumOffset=lineNumOffset,
                             defaults = list(type="video/mp4"))
      str = c(paste0('HTML("<video ',
                     ifelse(any(names(options)=="width"), paste0("width='", options$width, "' "), ""),
                     ifelse(any(names(options)=="height"), paste0("height='", options$height, "' "), ""),
                     ifelse(any(names(options)=="autoplay") && options$autoplay, "autoplay ", ""),
                     ' controls>",'),
              paste0("\"  <source src='", options$filename, "' type='", options$type, "'>\","),
              "\"  <p>Your browser does not support &lt;video&gt;</p>\",",
              "\"</video>\"),")
      rtn = c(rtn, str)
      
    } else stop(type, " is not a valid \\shinySomeCommand command")
  } # end for each command
  return(rtn)
} # end processCommands()
